(* glFramebuffer.ml    Framebuffer object extension of OpenGL ES
 *
 * http://khronos.org/registry/gles/extensions/OES/OES_framebuffer_object.txt
 *)
type renderbuffer_id = int32

let (renderbuffer0: renderbuffer_id) = 0l

type rendertarget = [ `renderbuffer ]

type renderfmt =
    (* For color; last 2 are optional (not available in all implementations) *)
    [ `rgb565 | `rgba4 | `rgb5_a1 | `rgba8 | `rgb8
    (* For depth; last 2 are optional *)
    | `depth_component16 | `depth_component24 | `depth_component32
    (* For stencil; all are optional *)
    | `stencil_index1 | `stencil_index4 | `stencil_index8
    ]

type renderparam =
    [ `width | `height | `red_size | `green_size | `blue_size
    | `alpha_size | `depth_size | `stencil_size
    ]
    
type renderparam_fmt = [ `internal_format ]

type id = int32 (* Framebuffer id *)

let (framebuffer0 : id) = 0l

type target = [ `framebuffer ]

type status =
    [ `complete | `incomplete_attachment | `incomplete_missing_attachment
    | `incomplete_dimensions | `incomplete_formats | `unsupported
    ]
    
type attachment = [ `color0 | `depth | `stencil ]

type textarget = [ `texture_2d ]

type attachtype =
    [ `none
    | `renderbuffer of renderbuffer_id
    | `texture of GlTex.texture_id
    ]

type enumeration =
    [ `renderbuffer ]

external is_renderbuffer : renderbuffer_id -> bool =
    "ml_glIsRenderbufferOES"
    
external bind_renderbuffer : rendertarget -> renderbuffer_id -> unit =
    "ml_glBindRenderbufferOES"
    
external delete_renderbuffer : renderbuffer_id -> unit =
    "ml_glDeleteRenderbuffer"

let delete_renderbuffers ids =
    Array.iter (fun id -> delete_renderbuffer id) ids

external gen_renderbuffer : unit -> renderbuffer_id =
    "ml_glGenRenderbuffer"

let gen_renderbuffers n =
    Array.init n (fun _ -> gen_renderbuffer ())
    
external renderbuffer_storage :
    rendertarget -> renderfmt -> int -> int -> unit =
    "ml_glRenderbufferStorageOES"

external get_renderbuffer_parameter : rendertarget -> renderparam -> int =
    "ml_glGetRenderbufferParameter"
    
external get_renderbuffer_parameter_fmt :
    rendertarget -> renderparam_fmt -> renderfmt =
    "ml_glGetRenderbufferParameter_fmt"

external is_framebuffer : id -> bool =
    "ml_glIsFramebufferOES"

external bind_framebuffer : target -> id -> unit =
    "ml_glBindFramebufferOES"

external delete_framebuffer : id -> unit =
    "ml_glDeleteFramebuffer"

let delete_framebuffers ids =
    Array.iter (fun id -> delete_framebuffer id) ids

external gen_framebuffer : unit -> id =
    "ml_glGenFramebuffer"

let gen_framebuffers n =
    Array.init n (fun _ -> gen_framebuffer ())

external check_status : target -> status =
    "ml_glCheckFramebufferStatus"

external renderbuffer :
    target -> attachment -> rendertarget -> renderbuffer_id -> unit =
    "ml_glFramebufferRenderbuffer"

external texture2d :
    target -> attachment -> textarget -> GlTex.texture_id -> int -> unit =
    "ml_glFramebufferTexture2D"

(* Simplified version of glGetFramebufferAttachmentParameter
 *)
external get_attachment_type : target -> attachment -> attachtype =
    "ml_glGetAttachmentType"
    
external generate_mipmap : textarget -> unit =
    "ml_glGenerateMipmapOES"
    
(* Translate an enumerated variant to its GL_XXX integer equivalent.
 *)
external code : enumeration -> int =
    "ml_glTranslateEnum"
